
import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;

/**
 * Created by IntelliJ IDEA.
 * User: Amir  Sheibani
 * Date: Mar 31, 2004
 * Time: 6:44:32 PM
 * To change this template use Options | File Templates.
 */
public class VideoFrame extends JFrame {

    public static final int FRAME_RATE = 100;
    public static final int WIDTH = 400;
    public static final int HEIGHT = 320;
    public static final int PADDING = 20;

    private GraphicComponent graphicComponent;
    private File file;

    JPanel fileChooserPanel;
    JPanel buttonPanel;

    public VideoFrame(File file, int width, int height, int padding, int frameRate) throws IOException {
        this(width, height, padding, frameRate);
        this.file = file;

    }

    public VideoFrame(int width, int height, int padding, int frameRate) throws IOException {

        graphicComponent = new GraphicComponent(width, height, padding, frameRate);
        init();
        setSize(2 * width + 3 * padding,
                HEIGHT + 3 * padding +
                (int) fileChooserPanel.getPreferredSize().getHeight() +
                (int) buttonPanel.getPreferredSize().getHeight());
        setLocation(100, 50);
        setVisible(true);
    }

    private void init() {

        final JButton fileChooserButton = new JButton("Load File");
        final JButton jumpBegButton = new JButton("<<");
        final JButton rewindButton = new JButton("<");
        final JButton playButton = new JButton("Play");
        final JButton pauseButton = new JButton("||");
        final JButton stopButton = new JButton("Stop");
        final JButton forwardButton = new JButton(">");
        final JButton jumpEndButton = new JButton(">>");

        jumpBegButton.setEnabled(false);
        rewindButton.setEnabled(false);
        playButton.setEnabled(false);
        pauseButton.setEnabled(false);
        stopButton.setEnabled(false);
        forwardButton.setEnabled(false);
        jumpEndButton.setEnabled(false);

        buttonPanel = new JPanel();
        buttonPanel.add(jumpBegButton);
        buttonPanel.add(rewindButton);
        buttonPanel.add(playButton);
        buttonPanel.add(pauseButton);
        buttonPanel.add(stopButton);
        buttonPanel.add(forwardButton);
        buttonPanel.add(jumpEndButton);

        fileChooserPanel = new JPanel();
        fileChooserPanel.setLayout(new BorderLayout());
        JPanel fileChooserWestPanel = new JPanel();
        fileChooserPanel.add(fileChooserWestPanel, BorderLayout.WEST);
        final JLabel fileLabel = new JLabel(" Current File: ");
        final JLabel loadTimeLabel = new JLabel();
        fileChooserPanel.add(loadTimeLabel, BorderLayout.EAST);
        fileChooserWestPanel.add(fileChooserButton);
        fileChooserWestPanel.add(fileLabel);

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(fileChooserPanel, BorderLayout.NORTH);
        getContentPane().add(graphicComponent, BorderLayout.CENTER);
        getContentPane().add(buttonPanel, BorderLayout.SOUTH);

        fileChooserButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                VideoFileChooser chooser = new VideoFileChooser(VideoFrame.this, "Select Video File");
                if (file != null) {
                    if (file.exists()) {
                        chooser.setSelectedFile(file);
                        chooser.setCurrentDirectory(file);
                    } else {
                        chooser.setCurrentDirectory(new File(".."));
                    }
                }

                try {
                    File newFile = chooser.open();
                    if (newFile != null) {
                        graphicComponent.reset();
                        file = newFile;
                        fileLabel.setText(" Current File: " + file.getName());
                        graphicComponent.load(file);
                        loadTimeLabel.setText("Load Time: " + graphicComponent.getLoadTime() + "ms ");
                        jumpBegButton.setEnabled(true);
                        rewindButton.setEnabled(true);
                        playButton.setEnabled(true);
                        pauseButton.setEnabled(true);
                        stopButton.setEnabled(true);
                        forwardButton.setEnabled(true);
                        jumpEndButton.setEnabled(true);
                    }
                } catch (IOException ioe) {
                    ioe.printStackTrace();
                }
            }
        });

        jumpBegButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                graphicComponent.jumpBeg();
            }
        });

        rewindButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                graphicComponent.rewind();
            }
        });
        playButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                graphicComponent.play();
            }
        });
        pauseButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                graphicComponent.pause();
            }
        });
        stopButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                graphicComponent.stop();
            }
        });
        forwardButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                graphicComponent.forward();
            }
        });
        jumpEndButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                graphicComponent.jumpEnd();
            }
        });

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                dispose();
                System.exit(0);
            }
        });
    }

    public static void main(String[] args) throws Exception {

        if (args.length == 0) {
            File file = new File("sample\\sample3_1.576v");
            new VideoFrame(file, WIDTH, HEIGHT, PADDING, FRAME_RATE);
        } else {
            new VideoFrame(WIDTH, HEIGHT, PADDING, FRAME_RATE);
        }
    }
}
