package model;

import motion.MotionVector;

import java.util.Stack;
import java.awt.*;

/**
 * Created by IntelliJ IDEA.
 * User: Administrator
 * Date: Apr 18, 2004
 * Time: 8:04:29 PM
 * To change this template use File | Settings | File Templates.
 */
public class Algorithm {
    //sizes of macroblocks and macroblock search space
    public static final int MBSIZE = 8;
    public static final int MVDIST = 24;

    //searchAlgo's
    public static final int EXHAUSTIVE = 1;
    public static final int LOGARITHMIC = 2;
    public static final int DIAMOND = 3;

    //diffAlgo's
    public static final int MAD = 1;
    public static final int MSD = 2;
    public static final int INTEGRAL = 3;

    //searchAlgo constants
    private static final int LOGARITHMIC_INIT_DIST = MVDIST/2;

    //testing similarity of motion vectors
    private static final double THRES_DIRECTION = 0.9;
    private static final double THRES_MAGNITUDE = 3*MVDIST/4;

    //testing similarity of macroblocks
    //if difference between macroblocks is less than this, then they are essentially same
    private static final int DIFF = 4; // this value should be dependent on diffAlgo
                                                       // currently, not important
    private static final int[] THRES_DIFFERENCE = { 0, DIFF /*MAD*/, DIFF*DIFF /*MSD*/, DIFF*2 /*INTEGRAL*/};

    //setting colors of objects
    private static final int COLOR_DIFFERENTIAL = 32;

    ImageRGBData img, prevImg;

    int width, height;
    int gridWidth, gridHeight;

    private int searchAlgo;
    private int diffAlgo;

    MotionVector mv[];
    int colorSet[];

    public int getNumObjects() {
        return numObjects;
    }

    public int getNumMvs() {
        return numMvs;
    }

    private int numMvs;
    private int numObjects;


    public Algorithm(int searchAlgo, int diffAlgo)
    {
        this.searchAlgo = searchAlgo;
        this.diffAlgo = diffAlgo;
    }

    public void runAlgo(ImageRGBData img, ImageRGBData prevImg, int[] rgb)
    {
        this.img = img;
        this.prevImg = prevImg;

        width = img.getWidth(); height = img.getHeight();
        gridWidth = width/MBSIZE; gridHeight = height/MBSIZE;

        // calculate motion vectors
        numMvs = 0;
        mv = new MotionVector[gridWidth * gridHeight];
        for (int j=0;j<gridHeight; j++)
            for (int i=0; i<gridWidth; i++)
            {
                mv[(j*gridWidth) + i] = calcMotionVector(i, j);
                if (mv[(j*gridWidth) + i].x !=0 ||
                        mv[(j*gridWidth) + i].x !=0 )
                    numMvs++;
            }

        //congregate them
        colorSet = new int[gridWidth*gridHeight];
        for (int i=0; i<gridHeight*gridWidth; i++)
            colorSet[i] = 0;

        Stack stack = new Stack();
        int cur_color = 1;
        int cur_putcolor;
        for (int j=0;j<gridHeight; j++)
            for (int i=0; i<gridWidth; i++)
                if (colorSet[(j*gridWidth) + i] == 0)
                {
                    //we havent visited this yet
                    // do a flood fill at this point from this location
                    // I dont think that there is any way other than this flood fill algo
                    stack.push(new Point(i,j));

                    //use this to put in colorSet a color from the group itself
                    cur_putcolor = (img.getData())[j*MBSIZE*width+i*MBSIZE];
                    //cur_putcolor = cur_color;

                    while (!stack.empty())
                    {
                        Point pt = (Point)stack.pop();

                        if (colorSet[pt.y * gridWidth + pt.x] != 0)
                            continue;

                        if (!test_similarity(mv[j*gridWidth+i], mv[pt.y*gridWidth+pt.x]))
                            continue;

                        colorSet[pt.y * gridWidth + pt.x] = cur_putcolor;

                        for (int p=-1; p<=1; p++)
                            for (int q=-1;q<=1; q++)
                            {
                                int u = pt.x + p; int v = pt.y + q;
                                if (u>=0 && u<gridWidth && v>=0 && v<gridHeight)
                                    if (colorSet[v * gridWidth + u] == 0)
                                        stack.push(new Point(u,v));
                            }
                    }

                    cur_color++;
                }

        numObjects = cur_color-1;


        //fill in rgb for display purposes
        for (int j=0;j<height; j++)
            for (int i=0; i<width; i++)
            {
                int t = j*width+i;
                int t2 = (j/MBSIZE)*gridWidth+(i/MBSIZE);


                // **** USEFUL FOR DEBUGGING
                // use this to mark off block which had non-zero motion vectors
                // without using the results of the seed-fill algo
                // to isolate problems in the finding motion vectors, and seed fill algo
                /*
                if (mv[t2].x != 0 || mv[t2].y != 0)
                    rgb[t] = 255 << 16 + 255 << 8 + 255;
                else
                    rgb[t] = 0; */


                //display each group by a different color
                /*rgb[t] = ((colorSet[t2]*COLOR_DIFFERENTIAL) << 16) +
                        ((colorSet[t2]*COLOR_DIFFERENTIAL) << 8) +
                        (colorSet[t2]*COLOR_DIFFERENTIAL);*/

                //directly display the putcolor
                rgb[t] = colorSet[t2];
            }
    }

    private boolean test_similarity(MotionVector mv1, MotionVector mv2)
    {
        //these atan and sqrt operations are slow, but dont matter much with Logarithmic search
        double direction1 = (mv1.x != 0)? Math.atan((double)mv1.y / mv1.x) : 0;
        double direction2 = (mv2.x != 0)? Math.atan((double)mv2.y / mv2.x) : 0;
        double magnitude1 = Math.sqrt(mv1.x*mv1.x + mv1.y*mv1.y);
        double magnitude2 = Math.sqrt(mv2.x*mv2.x + mv2.y*mv2.y);

        if (Math.abs(direction1-direction2) <= THRES_DIRECTION &&
                   Math.abs(magnitude1-magnitude2) <= THRES_MAGNITUDE )
            return true;
        else
            return false;
    }

    // calculate motion vector for the macroblock located at i,j in macroblock space
    private MotionVector calcMotionVector(int i, int j)
    {
        int inImgX = i*MBSIZE;
        int inImgY = j*MBSIZE;

        //calculate the space we will search in the previous frame
        int dist = MVDIST;

        int inPImgX1 = inImgX-dist;
        int inPImgX2 = inImgX+MBSIZE-1+dist;
        int inPImgY1 = inImgY-dist;
        int inPImgY2 = inImgY+MBSIZE-1+dist;
        if (inPImgX1<0) inPImgX1 = 0;
        if (inPImgY1<0) inPImgY1 = 0;
        if (inPImgX2>width-1) inPImgX2 = width-1;
        if (inPImgY2>height-1) inPImgY2 = height-1;

        //these are the solution vertices to be set by the algorithm
        int minX = inImgX, minY = inImgY;

        switch(searchAlgo)
        {
            case EXHAUSTIVE:
                {
                    int minDiff = 10000000;
                    for (int x=inPImgX1; x+MBSIZE-1 <= inPImgX2; x++)
                        for (int y=inPImgY1; y+MBSIZE-1 <= inPImgY2; y++)
                        {
                            int diff = diffMacroBlock(inImgX, inImgY, x, y);
                            if (diff < minDiff)
                            {
                                minDiff = diff;
                                minX = x;
                                minY = y;
                            }
                        }
                    break;
                }

            case LOGARITHMIC:
                {
                    int minDiff, diff;
                    int newX, newY;

                    int firstTime = 1;

                    //distance from center
                    dist = LOGARITHMIC_INIT_DIST;

                    //initial center is the original location
                    minX = inImgX; minY = inImgY;

                    while (dist>0)
                    {
                        minDiff = diffMacroBlock(inImgX, inImgY, minX, minY);
                        newX=minX; newY=minY;
                        if (firstTime == 1)
                        {
                            // if the direct block at inImgX and inImgY has low difference
                            // dont look for anything
                            firstTime = 0;
                            if (minDiff < (THRES_DIFFERENCE[diffAlgo]*MBSIZE*MBSIZE))
                                break;
                        }

                        for (int p=-1; p<=1; p++)
                            for (int q=-1;q<=1; q++)
                            {
                                int u = minX + p*dist; int v = minY + q*dist;
                                if (u>=inPImgX1 && (u+MBSIZE-1)<=inPImgX2 &&
                                        v>=inPImgY1 && (v+MBSIZE-1)<=inPImgY2)
                                {
                                    diff = diffMacroBlock(inImgX, inImgY, u, v);
                                    if (diff < minDiff)
                                    {
                                        minDiff = diff;
                                        newX = u;
                                        newY = v;
                                    }
                                }
                            }

                        if ((newX == minX) && (newY == minY))
                            dist = dist / 2; // if the best is the center, halve the distance

                        // relocate the center
                        minX = newX; minY = newY;
                    }

                    //when dist becomes zero, we exit the above while loop
                    // and at this stage, the minX and minY values are for the best match
                    // solution that the logarithmic method found.

                    break;
                }
            case DIAMOND:
                throw new UnsupportedOperationException();
                //break;
        }

        return new MotionVector(minX-inImgX, minY-inImgY);
    }

    private int diffMacroBlock(int x1, int y1, int x2, int y2)
    {
        int difference = 0;

        switch(diffAlgo)
        {
            case MAD:
                {
                    for (int i=0; i <MBSIZE; i++)
                        for(int j=0; j<MBSIZE; j++)
                        {
                            difference += Math.abs(img.getRed(x1+i,y1+j) - prevImg.getRed(x2+i,y2+j));
                        }
                    break;
                }

            case MSD:
                {
                    for (int i=0; i <MBSIZE; i++)
                        for(int j=0; j<MBSIZE; j++)
                        {
                            difference += ((img.getRed(x1+i,y1+j) - prevImg.getRed(x2+i,y2+j)) *
                                                (img.getRed(x1+i,y1+j) - prevImg.getRed(x2+i,y2+j)));
                        }
                    break;
                }
             case INTEGRAL:
                {
                    // INTEGRAL is untested
                    for (int i=0; i <MBSIZE; i++)
                    {
                        int colsum1 = 0, colsum2 = 0;

                        for (int j=0; j<MBSIZE; j++)
                        {
                            colsum1 += img.getRed(x1+i,y1+j);
                            colsum2 += prevImg.getRed(x2+i,y2+j);
                        }

                        difference += Math.abs(colsum1 - colsum2);
                    }

                    for (int j=0; j <MBSIZE; j++)
                        {
                        int rowsum1 = 0, rowsum2 = 0;

                        for (int i=0; i<MBSIZE; i++)
                            {
                            rowsum1 += img.getRed(x1+i,y1+j);
                            rowsum2 += prevImg.getRed(x2+i,y2+j);
                            }

                        difference += Math.abs(rowsum1 - rowsum2);
                        }
                }
        }

        return difference;
    }

}
